#include <DOS.H>
#include "I2C400.H"




unsigned char I2C_Freq=Clock_Freq_88;	// Default frequency for the configuration



int LeaveState (unsigned char State);   // Wait until the machine leave the state State



// I2C_Config : set the parameters for the PCA6564
// Set the frequency

void I2C_Config(unsigned char Freq)
{
	I2C_Freq=Freq;						// Store the frequency
	outportb (I2CCON,(inportb(I2CCON) & 0xF8) | I2C_Freq);		// Keep the old config and set the frequency
}




// I2C_Init : initialize the PCA9564
// Configure the bus at a given speed
//
// !!! Call I2c_Config before !!!
//
// The function return :
//  0 if the initialisation is successfull
// -1 if the state machine is not in the initial state after reset



int I2C_Init(void)
{
	outportb (Adr_Reset_0,0x00);		// Reset the PCA9564
	delay (2);							// Wait for 2 ms
	outportb (Adr_Reset_1,0x00);		// End of reset, the PCA9564 must be in intial state
	delay (2);							// Wait for 2 ms
	if (inportb (I2CSTA)!=0xF8)			// Check if the state machine is in the initial state
		return (-1);					// Initilization problem : return (-1)

	outportb (I2CCON,0x40 | I2C_Freq);		// ENSIO=1 	(Enable SIO)
										// Set the frequency
	return (0);							// Initialization successfull : return 0
}



// I2C_Read : Read an data at the adress Adress
//
// The function return :
//  0 if the transmission is successfull (2 acknoledges)
// -1 if the state machine is not in the initial state at the beginning of the function
// -2 transmission error while sending the START bit
// -3 no acknoledge after adress transmission, the line came back in the initial state
// -4 no acknoledge after adress transmission, the line didn't came back in the initial state
// -5 no acknoledge after adress transmission or another master took the line
// -6 transmission error while sending the START bitTransmitting the data
// -7 The state machine didn't return in the initial state after the sent of the stop bit


int I2C_Read(unsigned char Adress)
{
	unsigned char data;
	int TimeOut=100;

	// ::: State=0xF8 ::: (Initial condition)

	if (inportb (I2CSTA)!=0xF8)					// Check if the state machine is in the initial state
		return (-1);							// Initilization problem : return (-1)

//	outportb (I2CCON,inportb (I2CCON) | 0x80);	// Set the AA bit

	outportb (I2CCON,inportb (I2CCON) | 0x20);	// Send the start condition
	LeaveState(0xF8);							// Wait for the state machine to leave the current state
	if (inportb (I2CSTA)!=0x08) return (-2);	// Check if the state machine is in state 0x08

	// ::: State=0x08 ::: (Start sent)

	outportb (I2CDAT,Adress | 0x01);			// Write the slave adress (Set the LSB to 0)
	outportb (I2CCON,inportb (I2CCON) & 0xDF);	// Reset the start bit
	outportb (I2CCON,inportb (I2CCON) & 0xF7);	// Reset the SI bit
	LeaveState (0x08);							// Wait the new state

	switch (inportb (I2CSTA))                   // Analyse the new state
	{
		case 0x40 : break;                      // The slave acknoledge
		case 0x48 :								// No acknoledge
		{
			outportb (I2CCON,inportb (I2CCON) | 0x10);	// Send the stop condition
			LeaveState (0x48);					// Wait the new state
			outportb (I2CCON,inportb (I2CCON) & 0xEF);	// Reset the stop condition
			if (inportb(I2CSTA)==0xF8)			// Check if the machine came back in the initial state
				return (-3);					// Initial state : return -3
			else
				return (-4);					// An error accured : return -4
		}
		case 0x18 :
		{
			outportb (I2CCON,inportb (I2CCON) | 0x10);	// Send the stop condition
			LeaveState (0x18);					// Wait the new state
			outportb (I2CCON,inportb (I2CCON) & 0xEF);	// Reset the stop condition
			if (inportb(I2CSTA)==0xF8)			// Check if the machine came back in the initial state
				return (-3);					// Initial state : return -3
			else
				return (-4);					// An error accured : return -4
		}
		default   : return (-5);				// Another master took the line ?
	}

	// ::: State=0x40 ::: (Adress + acknoledge sent)

	outportb (I2CCON,inportb (I2CCON) & 0xF7);	// Reset the SI bit


	while (!(inportb(I2CCON) & 0x08))			// Wait for the SI bit to be set
	{
		if (TimeOut--==0) return (-6);
	}

	data=inportb  (I2CDAT);						// Read the data from the Data Register


	LeaveState (0x40);							// Wait the new state
	if (inportb (I2CSTA)!=0x58)					// Check if the state machine is in state 0x28
	{
		return (-7);							// Not in the right state, return (-6)
	}

	// ::: State=0x50 ::: (data + acknoledge sent)

	outportb (I2CCON,inportb (I2CCON) | 0x10);	// Send the stop condition
	LeaveState (0x58);					// Wait the new state
	outportb (I2CCON,inportb (I2CCON) & 0xEF);	// Reset the stop condition
	if (inportb(I2CSTA)!=0xF8)			// Check if the machine came back in the initial state
		return (-8);					// Not in the initial state : return -7


	return (data);							// Transmission was successfull : return 0
}



// I2C_Write : send a the byte Data at the adress Data on the I2C bus
//
// The function return :
//  0 if the transmission is successfull (2 acknoledges)
// -1 if the state machine is not in the initial state at the beginning of the function
// -2 transmission error while sending the START bit
// -3 no acknoledge after adress transmission, the line came back in the initial state
// -4 no acknoledge after adress transmission, the line didn't came back in the initial state
// -5 no acknoledge after adress transmission or another master took the line
// -6 transmission error while sending the START bitTransmitting the data
// -7 The state machine didn't return in the initial state after the sent of the stop bit


int I2C_Write(unsigned char Adress, unsigned char Data)
{

	// ::: State=0xF8 ::: (Initial condition)


	if (inportb (I2CSTA)!=0xF8)					// Check if the state machine is in the initial state
		return (-1);							// Initilization problem : return (-1)

	outportb (I2CCON,inportb (I2CCON) & 0x7F);	// Reset the AA bit
	outportb (I2CCON,inportb (I2CCON) | 0x20);	// Send the start condition
	LeaveState(0xF8);							// Wait for the state machine to leave the current state
	if (inportb (I2CSTA)!=0x08) return (-2);	// Check if the state machine is in state 0x08

	// ::: State=0x08 ::: (Start sent)

	outportb (I2CDAT,Adress & 0xFE);			// Write the slave adress (Set the LSB to 0)
	outportb (I2CCON,inportb (I2CCON) & 0xDF);	// Reset the start bit
	outportb (I2CCON,inportb (I2CCON) & 0xF7);	// Reset the SI bit
	LeaveState (0x08);							// Wait the new state

	switch (inportb (I2CSTA))                   // Analyse the new state
	{
		case 0x18 : break;                      // The slave acknoledge
		case 0x20 :
		{
			outportb (I2CCON,inportb (I2CCON) | 0x10);	// Send the stop condition
			LeaveState (0x20);					// Wait the new state
			outportb (I2CCON,inportb (I2CCON) & 0xEF);	// Reset the stop condition
			if (inportb(I2CSTA)==0xF8)			// Check if the machine came back in the initial state
				return (-3);					// Initial state : return -3
			else
				return (-4);					// An error accured : return -4
		}
		case 0x48 :
		{
			outportb (I2CCON,inportb (I2CCON) | 0x10);	// Send the stop condition
			LeaveState (0x20);					// Wait the new state
			outportb (I2CCON,inportb (I2CCON) & 0xEF);	// Reset the stop condition
			if (inportb(I2CSTA)==0xF8)			// Check if the machine came back in the initial state
				return (-3);					// Initial state : return -3
			else
				return (-4);					// An error accured : return -4
		}
		default   : return (-5);				// Another master took the line ?
	}

	// ::: State=0x18 ::: (Adress + acknoledge sent)

	outportb (I2CDAT,Data);						// Write the data in the Data Register
	outportb (I2CCON,inportb (I2CCON) & 0xF7);	// Reset the SI bit

	LeaveState (0x18);							// Wait the new state
	if (inportb (I2CSTA)!=0x28)					// Check if the state machine is in state 0x28
	{
		return (-6);							// Not in the right state, return (-6)
	}

	// ::: State=0x28 ::: (data + acknoledge sent)

	outportb (I2CCON,inportb (I2CCON) | 0x10);	// Send the stop condition
	LeaveState (0x28);					// Wait the new state
	outportb (I2CCON,inportb (I2CCON) & 0xEF);	// Reset the stop condition
	if (inportb(I2CSTA)!=0xF8)			// Check if the machine came back in the initial state
		return (-7);					// Not in the initial state : return -7

	return (0);							// Transmission was successfull : return 0
}



/******************************************************

					Local functions

*******************************************************/


// WaitState : Wait for the state machine to leave the current state (State)
// The fonction return :
// 	0 if the state machine reach a new state
// -1 if the timeout is over

int LeaveState (unsigned char State)
{
	int TimeOut=500;					// Preset the timeout
	while (inportb (I2CSTA)==State)		// CHeck the state
	{
		if (!TimeOut--) return (-1);   	// Decrement the time out and check if it is not to zero
	}
	return (0);							// The state is reach, return 0
}
